"use strict";
// ------------------------------------------------------------------------------
//  Copyright (c) 2017-present, RobotlegsJS. All Rights Reserved.
//
//  NOTICE: You are permitted to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
// ------------------------------------------------------------------------------
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@robotlegsjs/core");
var AsyncCommand_1 = require("./AsyncCommand");
var SubCommandMapping_1 = require("./SubCommandMapping");
var SubCommandMappingList_1 = require("./SubCommandMappingList");
var SubCommandPayload_1 = require("./SubCommandPayload");
var AbstractMacro = /** @class */ (function (_super) {
    tslib_1.__extends(AbstractMacro, _super);
    function AbstractMacro(context, injector) {
        var _this = _super.call(this, context) || this;
        _this._commandPayloads = [];
        _this._injector = injector.createChild();
        _this._macroPayload = new core_1.CommandPayload();
        _this._mappings = new SubCommandMappingList_1.SubCommandMappingList();
        return _this;
    }
    AbstractMacro.prototype.add = function (commandClass) {
        var mapping = new SubCommandMapping_1.SubCommandMapping(commandClass);
        this._mappings.addMapping(mapping);
        return mapping;
    };
    AbstractMacro.prototype.remove = function (commandClass) {
        this._mappings.removeMappingsFor(commandClass);
    };
    AbstractMacro.prototype.captureMacroPayload = function (executeArguments) {
        this._executeArguments = executeArguments;
        for (var _i = 0, executeArguments_1 = executeArguments; _i < executeArguments_1.length; _i++) {
            var argument = executeArguments_1[_i];
            if (argument instanceof SubCommandPayload_1.SubCommandPayload) {
                this._commandPayloads.push(argument);
            }
            else {
                this._macroPayload.addPayload(argument, argument.constructor);
            }
        }
    };
    AbstractMacro.prototype.executeCommand = function (mapping) {
        var command;
        var commandClass = mapping.commandClass;
        var payloads = this._commandPayloads.concat(mapping.payloads);
        var hasPayloads = payloads.length > 0;
        this.mapMacroPayload(this._macroPayload);
        if (hasPayloads) {
            this.mapPayloads(payloads);
        }
        if (mapping.guards.length === 0 || core_1.guardsApprove(mapping.guards, this._injector)) {
            command = mapping.getOrCreateCommandInstance(this._injector);
            if (mapping.hooks.length > 0) {
                this._injector.bind(commandClass).toConstantValue(command);
                core_1.applyHooks(mapping.hooks, this._injector);
                this._injector.unbind(commandClass);
            }
        }
        if (hasPayloads) {
            this.unmapPayloads(payloads);
        }
        this.unmapMacroPayload(this._macroPayload);
        if (command) {
            var isAsync = command.constructor.prototype.registerCompleteCallback !== undefined;
            if (isAsync) {
                command.registerCompleteCallback(this.commandCompleteHandler.bind(this));
            }
            if (payloads.length > 0) {
                command.execute.apply(command, payloads.concat(this._executeArguments));
            }
            else {
                command.execute.apply(command, this._executeArguments);
            }
            if (!isAsync) {
                this.commandCompleteHandler(true);
            }
        }
        else {
            this.commandCompleteHandler(true);
        }
    };
    AbstractMacro.prototype.mapMacroPayload = function (payload) {
        var i = payload.length;
        while (i--) {
            this._injector.bind(payload.classes[i]).toConstantValue(payload.values[i]);
        }
    };
    AbstractMacro.prototype.unmapMacroPayload = function (payload) {
        var i = payload.length;
        while (i--) {
            this._injector.unbind(payload.classes[i]);
        }
    };
    AbstractMacro.prototype.mapPayloads = function (payloads) {
        this._commandPayloadsModule = new core_1.ContainerModule(function (bind, unbind) {
            payloads.forEach(function (payload) {
                if (payload.name.length > 0) {
                    bind(payload.type)
                        .toConstantValue(payload.data)
                        .whenTargetNamed(payload.name);
                }
                else {
                    bind(payload.type).toConstantValue(payload.data);
                }
            });
        });
        this._injector.load(this._commandPayloadsModule);
    };
    AbstractMacro.prototype.unmapPayloads = function (payloads) {
        this._injector.unload(this._commandPayloadsModule);
    };
    AbstractMacro = tslib_1.__decorate([
        core_1.injectable(),
        tslib_1.__param(0, core_1.inject(core_1.IContext)), tslib_1.__param(1, core_1.inject(core_1.IInjector)),
        tslib_1.__metadata("design:paramtypes", [Object, Object])
    ], AbstractMacro);
    return AbstractMacro;
}(AsyncCommand_1.AsyncCommand));
exports.AbstractMacro = AbstractMacro;
//# sourceMappingURL=AbstractMacro.js.map