"use strict";
// ------------------------------------------------------------------------------
//  Copyright (c) 2017-present, RobotlegsJS. All Rights Reserved.
//
//  NOTICE: You are permitted to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
// ------------------------------------------------------------------------------
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var core_1 = require("@robotlegsjs/core");
var AbstractMacro_1 = require("./AbstractMacro");
var SequenceMacro = /** @class */ (function (_super) {
    tslib_1.__extends(SequenceMacro, _super);
    function SequenceMacro() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SequenceMacro.prototype, "atomic", {
        get: function () {
            return this._customAtomic !== undefined ? this._customAtomic : true;
        },
        set: function (value) {
            if (!this._running && !this._completed) {
                this._customAtomic = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    SequenceMacro.prototype.execute = function (payload) {
        var payloads = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            payloads[_i - 1] = arguments[_i];
        }
        this.captureMacroPayload(arguments);
        this.prepare();
        this._atomic = this.atomic;
        this._success = true;
        this._running = true;
        this._completed = false;
        this._executionIndex = 0;
        this._commands = this._mappings.getList();
        this.executeNext();
    };
    SequenceMacro.prototype.executeNext = function () {
        if (this.hasCommands) {
            var mapping = this._commands[this._executionIndex++];
            this.executeCommand(mapping);
        }
        else {
            this.dispatchComplete(this._success);
        }
    };
    Object.defineProperty(SequenceMacro.prototype, "hasCommands", {
        get: function () {
            return this._commands && this._executionIndex < this._commands.length;
        },
        enumerable: true,
        configurable: true
    });
    SequenceMacro.prototype.commandCompleteHandler = function (success) {
        this._success = this._success && success;
        if (this._atomic || this._success) {
            this.executeNext();
        }
        else {
            this.dispatchComplete(false);
        }
    };
    SequenceMacro.prototype.dispatchComplete = function (success) {
        this._running = false;
        this._completed = true;
        this._success = true;
        this._executionIndex = 0;
        this._commands = null;
        _super.prototype.dispatchComplete.call(this, success);
    };
    SequenceMacro = tslib_1.__decorate([
        core_1.injectable()
    ], SequenceMacro);
    return SequenceMacro;
}(AbstractMacro_1.AbstractMacro));
exports.SequenceMacro = SequenceMacro;
//# sourceMappingURL=SequenceMacro.js.map